#include <iostream>
#include <math.h>

using namespace std;

#include "charset2.hpp"

/**************************************************************
Die Funktion void init(const char *) bernimmt als Parameter
einen String, dessen einzelne Zeichen Elemente der Menge sein
sollen. Wird init() ohne Argument aufgerufen, dann soll eine
leere Menge erzeugt werden.
**************************************************************/

CharSet::CharSet(const char *pinitString)
{
	int index;
	int position;
	int dez;

	for (int i = 0; i < ARRSIZE; i++)
		data[i]=0;
	while (*pinitString != '\0')
	{
		index = ((int)(*pinitString)) / INTBITS;
		position = ((int)(*pinitString)) % INTBITS;
		dez = int(pow(2,position));
		data[index] = data[index] + dez;
		++pinitString;
	}
}

/**************************************************************
Die Funktion bool contains(unsigned char c) berprft, ob das
Zeichen c in der aktuellen Menge enthalten ist.
**************************************************************/

bool CharSet::contains(unsigned char str) const
{
	return (data[str / INTBITS] & 1 << (str % INTBITS)) != 0;
}

/**************************************************************
Der Operator + bildet die Vereinigungsmenge von zwei Mengen.
Jedes Element  data[i] des Ergebnisobjekts entsteht durch
bitweise Or-Verknpfung von  data[i] der beiden Operanden.
**************************************************************/

CharSet CharSet::operator+(const CharSet &string) const
{
	CharSet help;

	for (int i = 0; i < ARRSIZE; i++)
		help.data[i] = data[i] | string.data[i];

	return help;
}

/**************************************************************
Der Operator - bildet die Restmenge von zwei Mengen. Jedes
Element data[i] des Ergebnisobjekts entsteht durch bitweise
And-Verknpfung von  data[i] des linken Operanden mit dem
bitweise invertierten Element  ~data[i] des rechten Operanden.
**************************************************************/

CharSet CharSet::operator-(const CharSet &string) const
{
	CharSet help;

	for (int i = 0; i < ARRSIZE; i++)
		help.data[i] = data[i] & ~string.data[i];

	return help;
}

/**************************************************************
Der Operator * bildet die Schnittmenge von zwei Mengen. Jedes
Element  data[i] des Ergebnisobjekts entsteht durch bitweise
And-Verknpfung von  data[i] der beiden Operanden.
**************************************************************/

CharSet CharSet::operator*(const CharSet &string) const
{

	CharSet help;

	for (int i = 0; i < ARRSIZE; i++)
		help.data[i] = data[i] & string.data[i];

	return help;
}

/**************************************************************
Der Operator <= berprft, ob der linke Operand Teilmenge des
rechten Operanden ist. Der Vergleich ist wahr, wenn die
And-Verknpfung von  data[i] der beiden Operanden mit data[i]
des linken Operanden fr alle Arrayelemente identisch sind.
**************************************************************/

bool CharSet::operator<=(const CharSet &string) const
{
	CharSet help;

	for (int i = 0; i < ARRSIZE; i++)
	{
		help.data[i] = data[i] & string.data[i];
		if (help.data[i] != data [i])
			return false;
	}
	return true;
}

/**************************************************************
Der Operator >= berprft, ob der linke Operand Obermenge des
rechten Operanden ist.
Hinweis: Der Ausdruck a >= b hat den gleichen Wert wie b <= a.
Rufen Sie intern den (schon implementierten) Operator <= mit
vertauschten Operanden auf und vermeiden Sie auf diese Weise
Codeduplizierung.
**************************************************************/

bool CharSet::operator>=(const CharSet &string) const
{
	return string <= *this;
}

/**************************************************************
Der Operator == berprft, ob zwei Mengen identisch sind. Der
Vergleich ist wahr, wenn smtliche Elemente data[i] beider
Operanden identisch sind.
**************************************************************/

bool CharSet::operator==(const CharSet &string) const
{
	for (int i = 0; i < ARRSIZE; i++)
	{
		if (data[i] != string.data[i])
			return false;
	}
	return true;
}

/**************************************************************
Der Operator != berprft, ob zwei Mengen ungleich sind.
Hinweis: Der Ausdruck a != b hat den gleichen Wert wie
!(a == b). Rufen Sie intern den (schon implementierten)
Operator == auf und vermeiden Sie auf diese Weise Code-
duplizierung.
**************************************************************/

bool CharSet::operator!=(const CharSet &string) const
{
	return !(*this == string);
}

/**************************************************************
Die Funktion void print() gibt die in der Menge enthaltenen
Zeichen als lckenlosen String aus.
**************************************************************/

ostream &operator<<(ostream &out, const CharSet &string)
{
	for( int i = 0; i < string.NUMCHARS; i++)
	{
		if( string.contains((char)i) )
			out << (char) i;
	}
	return out; 
}