/**********************************************************************\
* Dateiname: tconsll.c
* Autor    : Jesorsky
* Projekt  : Programmieruebung
*
* Kurzbeschreibung: Diese Datei enthaelt die Funktionen fuer den Zugriff
* auf einen Container, in dem Objekte des Typs T aufbewahrt werden.
* Der Container wird als einfach verkettete Liste implementiert
* (ausfuehrliche Beschreibung in cont.htm.
*
* Datum:     Autor:      Grund der Aenderung:
* 24.09.1999 Jesorsky    Neuerstellung
* 19.10.1999 Jesorsky    Leere Parameterlisten ...() durch ...(void) 
*                        ersetzt
* 27.10.1999 Jesorsky    Kurzbeschreibung von pop_xxx, front() und
*                        back praezisiert
* 07.09.2000 Jesorsky    Dateinamen geaendert (urspr. TCont2.c)
* 18.08.2002 Jesorsky    Kommentare modifiziert
\**********************************************************************/

/*--- #includes der Form <...> ---------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

/*--- #includes der Form "..." ---------------------------------------*/

#include "t.h"
#include "tcon.h"

/*--- #define-Konstanten und Makros ----------------------------------*/

/*--- Datentypen (typedef) -------------------------------------------*/

typedef struct tagNODE
{
	T data;
	struct tagNODE *pNext;
} NODE;

/*--- Globale Konstanten ---------------------------------------------*/

/*--- Globale Variablen ----------------------------------------------*/

/*--- Modullokale Konstanten -----------------------------------------*/

/*--- Modullokale Variablen ------------------------------------------*/

static NODE *pHead = NULL;
static NODE *pTail;

/*--- Prototypen modullokaler Funktionen -----------------------------*/

static void checkMem(void *pv);

/*--- Funktionsdefinitionen ------------------------------------------*/

/**********************************************************************\
* Funktionsname: push_front
*
* Kurzbeschreibung: kopiert das Objekt *pItem an den Anfang des 
* Containers.
*
* Die modullokale Variable pHead wird veraendert, ggf. (bei leerem
* Container) auch die modullokale Variable pTail.
*
* Parameter:
* pItem     Adresse des zu uebernehmenden Objekts
\**********************************************************************/

void push_front(const T *pItem)
{
	NODE *pNeu;

	pNeu = (NODE *)malloc(sizeof(NODE));

	checkMem(pNeu);

	(*pNeu).data = *pItem;
	(*pNeu).pNext = pHead;

	if (pHead == NULL)
		pTail = pNeu;
	else
		;

	pHead = pNeu;
}

/**********************************************************************\
* Funktionsname: pop_front
*
* Kurzbeschreibung: entfernt das 1. Objekt aus dem Container. Es muss
* vorher vom Aufrufer mit empty() ueberprueft werden, dass mindestens
* ein Objekt vorhanden ist.

* Die modullokale Variable pHead wird veraendert.
\**********************************************************************/

void pop_front(void)
{
	NODE *pHelp;

	assert(pHead != NULL);

	pHelp = pHead;
	pHead = (*pHead).pNext;
	free(pHelp);
}

/**********************************************************************\
* Funktionsname: push_back
*
* Kurzbeschreibung: kopiert das Objekt *pItem an das Ende des 
* Containers.
*
* Die modullokale Variable pTail wird veraendert, ggf. (bei leerem
* Container) auch die modullokale Variable pHead.
*
* Parameter:
* pItem     Adresse des zu uebernehmenden Objekts
\**********************************************************************/

void push_back(const T *pItem)
{
	NODE *pNeu;

	pNeu = (NODE *)malloc(sizeof(NODE));

	checkMem(pNeu);

	(*pNeu).data = *pItem;
	(*pNeu).pNext = NULL;
	if (pHead == NULL)
		pHead = pNeu;
	else
		(*pTail).pNext = pNeu;

	pTail = pNeu;
}

/**********************************************************************\
* Funktionsname: pop_back
*
* Kurzbeschreibung: entfernt das letzte Objekt aus dem Container. Es 
* muss vorher vom Aufrufer mit empty() ueberprueft werden, dass 
* mindestens ein Objekt vorhanden ist.
*
* Die modullokale Variable pTail wird veraendert, ggfs (bei leerem
* Container) auch die modullokale Variable pHead.
\**********************************************************************/

void pop_back(void)
{
	NODE *pHelp;

	assert(pHead != NULL);

	if (pHead == pTail)   
	{
		pHead = NULL;
		free (pTail);
	}
	else
	{
		pHelp = pHead;
		while ((*pHelp).pNext != pTail)
		{
			 ;
			pHelp = (*pHelp).pNext;
		}
		pTail = pHelp;
		(*pTail).pNext = NULL;
	}
}

/**********************************************************************\
* Funktionsname: front
*
* Kurzbeschreibung: gestattet den Zugriff auf das erste Objekt des
* Containers (ohne es zu entfernen). Es muss vorher vom Aufrufer mit 
* empty() ueberprueft werden, dass mindestens ein Objekt vorhanden ist.
*
* Rueckgabewert:
* Adresse des ersten Container-Objekts
\**********************************************************************/

T *front(void)
{
	T *pFront;

	assert(pHead != NULL);

	pFront = &(*pHead).data;
	return pFront;
}

/**********************************************************************\
* Funktionsname: back
*
* Kurzbeschreibung: gestattet den Zugriff auf das letzte Objekt des
* Containers (ohne es zu entfernen). Es muss vorher vom Aufrufer mit 
* empty() ueberprueft werden, dass mindestens ein Objekt vorhanden ist.
*
* Rueckgabewert:
* Adresse des letzten Container-Objekts
\**********************************************************************/

T *back(void)
{
	T *pBack;

	assert(pHead != NULL);
	
	pBack = &(*pTail).data;
	return pBack;
}

/**********************************************************************\
* Funktionsname: empty
*
* Kurzbeschreibung: prueft, ob der Container leer ist.
*
* Rueckgabewert:
* 1   Container leer
* 0   Container enthaelt mindestens 1 Objekt
\**********************************************************************/

int empty(void)
{
	if (pHead == NULL)
		return 1;
	else
		return 0;
}

/**********************************************************************\
* Funktionsname: begin
*
* Kurzbeschreibung: liefert einen Zeiger auf das erste Objekt des
* Containers (ohne es zu entfernen). Es muss *nicht* geprueft werden, 
* ob der Container leer ist.
*
* Rueckgabewert:
* Adresse des ersten Container-Objekts   wenn Container nicht leer
* Funktionswert von end()                wenn Container leer
\**********************************************************************/

T *begin(void)
{
	if (pHead == NULL)
		return NULL;
	else
	{
		T *pHelp;
		pHelp = &(*pHead).data;
		return pHelp;
	}
}

/**********************************************************************\
* Funktionsname: end
*
* Kurzbeschreibung: liefert einen Zeiger auf das (fiktive) Objekt
* zurueck, das auf das letzte tatsaechlich vorhandene Objekt folgen
* wuerde. Der Wert muss mit dem Funktionswert der Funktion next() ueber-
* einstimmen, wenn man dieser Funktion einen Zeiger auf das *letzte*
* Element uebergibt. Bei leerem Container muss der Funktionswert mit
* dem der Funktion begin() uebereinstimmen.
*
* Rueckgabewert:
* Adresse des (fiktiven) Objekts *nach* dem letzten Container-Objekt
\**********************************************************************/

T *end(void)
{
	return NULL;
}

/**********************************************************************\
* Funktionsname: next
*
* Kurzbeschreibung: liefert, ausgehend vom aktuellen Objekt *pItem, 
* einen Zeiger auf das naechste Objekt des Containers zurueck. Es muss
* *nicht* geprueft werden, ob noch ein weiteres Objekt vorhanden ist.
*
* Parameter:
* pItem     Adresse des aktuellen Objekts
*
* Rueckgabewert:
* Adresse des naechsten Container-Objekts   wenn vorhanden
* Funktionswert von end()                   wenn nicht vorhanden
\**********************************************************************/

T *next(T *pItem)
{
	T *pHelp;
	pHelp = &((NODE *)pItem)->pNext->data;
	return pHelp;
}

/**********************************************************************\
* Funktionsname: checkMem
*
* Kurzbeschreibung: beendet das Programm mit einer Fehlermeldung,
* wenn ein NULL-Pointer uebergeben wird.
*
* Parameter:
* pv     zu prfende Adresse
\**********************************************************************/

void checkMem(void *pv)
{
	if (pv == NULL)
	{
		fprintf(stderr, "Out Of Memory\n");
		exit(1);
	}
}