/**********************************************************************\
* Dateiname: tconarr.c
* Autor    : Jesorsky
* Projekt  : cont2
*
* Kurzbeschreibung: Diese Datei enthaelt die Funktionen fuer den Zugriff
* auf einen Container, in dem Objekte des Typs T aufbewahrt werden.
* Der Container wird als Array implementiert
* (ausfuehrliche Beschreibung in cont.htm.
*
* Datum:     Autor:      Grund der Aenderung:
* 24.09.1999 Jesorsky    Neuerstellung
* 19.10.1999 Jesorsky    Leere Parameterlisten ...() durch ...(void) 
*                        ersetzt
* 27.10.1999 Jesorsky    Kurzbeschreibung von pop_xxx, front() und
*                        back praezisiert
* 07.09.2000 Jesorsky    Dateinamen geaendert (urspr. TCont2.c)
* 18.08.2002 Jesorsky    Kommentare modifiziert
\**********************************************************************/

/*--- #includes der Form <...> ---------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

/*--- #includes der Form "..." ---------------------------------------*/

#include "t.h"
#include "tcon.h"

/*--- #define-Konstanten und Makros ----------------------------------*/

/*--- Datentypen (typedef) -------------------------------------------*/

/*--- Globale Konstanten ---------------------------------------------*/

/*--- Globale Variablen ----------------------------------------------*/

/*--- Modullokale Konstanten -----------------------------------------*/

static const int array_size = 10;

/*--- Modullokale Variablen ------------------------------------------*/

static T *pData = NULL;
static int capacity = 0;
static int count = 0;

/*--- Prototypen modullokaler Funktionen -----------------------------*/

static void checkMem(void *pv);

/*--- Funktionsdefinitionen ------------------------------------------*/

/**********************************************************************\
* Funktionsname: push_front
*
* Kurzbeschreibung: kopiert das Objekt *pItem an den Anfang des 
* Containers.
*
* count wird inkrementiert
* capacity und pData werden verndert
*
* Parameter:
* pItem     Adresse des zu uebernehmenden Objekts
\**********************************************************************/

void push_front(const T *pItem)
{
	int cap_help = 0;
	int size_help = 0;

	if (count == capacity)
	{
		capacity = capacity + array_size;
		cap_help = capacity * sizeof(T);
		pData = (T *)realloc(pData, cap_help);
		checkMem(pData);
	}
	else
		;

	size_help = (count++) * sizeof(T);
	memcpy((pData + 1), pData, size_help);

	pData[0] = *pItem;
}

/**********************************************************************\
* Funktionsname: pop_front
*
* Kurzbeschreibung: entfernt das 1. Objekt aus dem Container. Es muss
* vorher vom Aufrufer mit empty() ueberprueft werden, dass mindestens
* ein Objekt vorhanden ist.

* count wird dekrementiert
\**********************************************************************/

void pop_front(void)
{
	int size_help = 0;

	assert(count != 0);

	size_help = (count--) * sizeof(T);
	memcpy(pData, (pData + 1), size_help);
}

/**********************************************************************\
* Funktionsname: push_back
*
* Kurzbeschreibung: kopiert das Objekt *pItem an das Ende des 
* Containers.
*
* count wird inkrementiert
* capacity und pData werden verndert
*
* Parameter:
* pItem     Adresse des zu uebernehmenden Objekts
\**********************************************************************/

void push_back(const T *pItem)
{
	int cap_help = 0;

	if (count == capacity)
	{
		capacity = capacity + array_size;
		cap_help = capacity * sizeof(T);
		pData = (T *)realloc(pData, cap_help);
		checkMem(pData);
	}
	pData[count++] = *pItem;
}

/**********************************************************************\
* Funktionsname: pop_back
*
* Kurzbeschreibung: entfernt das letzte Objekt aus dem Container. Es 
* muss vorher vom Aufrufer mit empty() ueberprueft werden, dass 
* mindestens ein Objekt vorhanden ist.
*
* count wird dekrementiert
\**********************************************************************/

void pop_back(void)
{
	assert(count != 0);

	count--;
}

/**********************************************************************\
* Funktionsname: front
*
* Kurzbeschreibung: gestattet den Zugriff auf das erste Objekt des
* Containers (ohne es zu entfernen). Es muss vorher vom Aufrufer mit 
* empty() ueberprueft werden, dass mindestens ein Objekt vorhanden ist.
*
* Rueckgabewert:
* Adresse des ersten Container-Objekts
\**********************************************************************/

T *front(void)
{
	assert(count != 0);

	return pData;
}

/**********************************************************************\
* Funktionsname: back
*
* Kurzbeschreibung: gestattet den Zugriff auf das letzte Objekt des
* Containers (ohne es zu entfernen). Es muss vorher vom Aufrufer mit 
* empty() ueberprueft werden, dass mindestens ein Objekt vorhanden ist.
*
* Rueckgabewert:
* Adresse des letzten Container-Objekts
\**********************************************************************/

T *back(void)
{
	T *back;

	assert(count != 0);

	back = pData + count - 1;

	return back;
}

/**********************************************************************\
* Funktionsname: empty
*
* Kurzbeschreibung: prueft, ob der Container leer ist.
*
* Rueckgabewert:
* 1   Container leer
* 0   Container enthaelt mindestens 1 Objekt
\**********************************************************************/

int empty(void)
{
	if (count == 0)
		return 1;
	else
		return 0;
}

/**********************************************************************\
* Funktionsname: begin
*
* Kurzbeschreibung: liefert einen Zeiger auf das erste Objekt des
* Containers (ohne es zu entfernen). Es muss *nicht* geprueft werden, 
* ob der Container leer ist.
*
* Rueckgabewert:
* Adresse des ersten Container-Objekts   wenn Container nicht leer
* Funktionswert von end()                wenn Container leer
\**********************************************************************/

T *begin(void)
{
	if (count == 0)
		return pData + count;
	else
		return pData;
}

/**********************************************************************\
* Funktionsname: end
*
* Kurzbeschreibung: liefert einen Zeiger auf das (fiktive) Objekt
* zurueck, das auf das letzte tatsaechlich vorhandene Objekt folgen
* wuerde. Der Wert muss mit dem Funktionswert der Funktion next() ueber-
* einstimmen, wenn man dieser Funktion einen Zeiger auf das *letzte*
* Element uebergibt. Bei leerem Container muss der Funktionswert mit
* dem der Funktion begin() uebereinstimmen.
*
* Rueckgabewert:
* Adresse des (fiktiven) Objekts *nach* dem letzten Container-Objekt
\**********************************************************************/

T *end(void)
{
	T *end;
	end = pData + count;
	return end;
}

/**********************************************************************\
* Funktionsname: next
*
* Kurzbeschreibung: liefert, ausgehend vom aktuellen Objekt *pItem, 
* einen Zeiger auf das naechste Objekt des Containers zurueck. Es muss
* *nicht* geprueft werden, ob noch ein weiteres Objekt vorhanden ist.
*
* Parameter:
* pItem     Adresse des aktuellen Objekts
*
* Rueckgabewert:
* Adresse des naechsten Container-Objekts   wenn vorhanden
* Funktionswert von end()                   wenn nicht vorhanden
\**********************************************************************/

T *next(T *pItem)
{
	return pItem + 1;
}

/**********************************************************************\
* Funktionsname: checkMem
*
* Kurzbeschreibung: beendet das Programm mit einer Fehlermeldung,
* wenn ein NULL-Pointer uebergeben wird.
*
* Parameter:
* pv     zu prfende Adresse
\**********************************************************************/

void checkMem(void *pv)
{
	if (pv == NULL)
	{
		fprintf(stderr, "Out Of Memory\n");
		exit(1);
	}
}