#ifndef CONARR_HPP
#define CONARR_HPP

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

template <class T>
class Cont
{
private:
	T *pData;
	int capacity;
	int count;
	Cont (const Cont &); //Kopier-Konstruktor
	Cont &operator = (const Cont &); //Zuweisung
public:
	class iterator
	{
	private:
		T *pElem;
	public:
		iterator(T *pElement = NULL) //Konstruktor
		{
			pElem = pElement;
		}
		iterator operator++(int) //T *next(T *) const;
		{
			return iterator(pElem++);
		}
		T &operator*() const //Dereferenzierungsoperator
		{
			return *pElem;
		}
		bool operator!=(const iterator &it) const //Vergleichsoperator
		{
			return pElem != it.pElem;
		}
	};
public:
	Cont(); //Konstruktor
	~Cont(); //Destruktor
	void push_front(const T&); //void push_front(const T *);
	void pop_front();
	void push_back(const T&); //void push_back(const T *);
	void pop_back();
	T &front() const; //T *front() const;
	T &back() const; //T *back() const;
	bool empty() const; //int empty() const;
	iterator begin() const //T *begin() const;
	{
		return iterator(pData);
	}
	iterator end() const //T *end() const;
	{
		return iterator(pData + count);
	}
	void checkMem (void *pv);
};

template <class T>
Cont<T>::Cont()
{
	count = 0;
	capacity = 0;
	pData = NULL;
}

template <class T>
Cont<T>::~Cont()
{
	free(pData);
}

template <class T>
void Cont<T>::push_front(const T &pItem)
{
	int cap_help = 0;
	int size_help = 0;

	if (count == capacity)
	{
		capacity = capacity + array_size;
		cap_help = capacity * sizeof(T);
		pData = (T *)realloc(pData, cap_help);
		checkMem(pData);
	}
	else
		;

	size_help = (count++) * sizeof(T);
	memcpy((pData + 1), pData, size_help);

	pData[0] = pItem;
}

template <class T>
void Cont<T>::pop_front(void)
{
	int size_help = 0;

	assert(count != 0);

	size_help = (count--) * sizeof(T);
	memcpy(pData, (pData + 1), size_help);
}

template <class T>
void Cont<T>::push_back(const T &pItem)
{
	int cap_help = 0;

	if (count == capacity)
	{
		capacity = capacity + array_size;
		cap_help = capacity * sizeof(T);
		pData = (T *)realloc(pData, cap_help);
		checkMem(pData);
	}
	pData[count++] = pItem;
}

template <class T>
void Cont<T>::pop_back()
{
	assert(count != 0);

	count--;
}

template <class T>
T &Cont<T>::front() const
{
	assert(count != 0);

	return pData[0];
}

template <class T>
T &Cont<T>::back() const
{
	assert(count != 0);
	return pData[count - 1];
}

template <class T>
bool Cont<T>::empty() const
{
	return count == 0;
}

template <class T>
void Cont<T>::checkMem(void *pv)
{
	if (pv == NULL)
	{
		fprintf(stderr, "Out Of Memory\n");
		exit(1);
	}
}

#endif //CONARR_HPP